return function(button_list, stepstype)
	-- Change tap_graphic to one of the judgment graphic names ripped from Simply Love.
	local tap_graphic= "Love"
	-- If the graphic has two frames for each judgment, like "-excellent" and
	-- "excellent-", then tap_frames_per_quant should be 2, to make the note
	-- animate between them.
	local tap_frames_per_quant= 2
	-- Each entry in the table is the number of times per beat that a
	-- quantization occurs.  This tells the engine that the first two frames
	-- (tap_frames_per_quant) are for the quantization that occurs once per
	-- beat (4ths).  The next two are for the quantization that occurs twice
	-- per beat (8ths).  3 times per beat is 12ths, and so on.  Since judgment
	-- graphics only have 6 judgments, this stops at 32nds, but extending it to
	-- 48ths, 64ths, 96ths, and 192nds is easy.
	local tap_state_map= NewSkin.generic_state_map(tap_frames_per_quant, {1, 2, 3, 4, 6, 8})
	-- Set rotate_taps to false if you want the taps to be right side up in all
	-- columns instead of rotated.
	local rotate_taps= true
	-- For explanations of everything else, read the default noteskin, it's a
	-- straight copy paste.
	local rots= {Left= 90, Down= 0, Up= 180, Right= 270}
	local hold_flips= {
		Left= "TexCoordFlipMode_None", Right= "TexCoordFlipMode_X",
		Down= "TexCoordFlipMode_None", Up= "TexCoordFlipMode_Y",
	}
	local roll_flips= {
		Left= "TexCoordFlipMode_None", Right= "TexCoordFlipMode_X",
		Down= "TexCoordFlipMode_None", Up= "TexCoordFlipMode_None",
	}
	local rev_hold_flips= {
		Left= "TexCoordFlipMode_None", Right= "TexCoordFlipMode_X",
		Down= "TexCoordFlipMode_Y", Up= "TexCoordFlipMode_None",
	}
	local rev_roll_flips= {
		Left= "TexCoordFlipMode_None", Right= "TexCoordFlipMode_X",
		Down= "TexCoordFlipMode_Y", Up= "TexCoordFlipMode_Y",
	}
	local hold_buttons= {
		Left= "left", Right= "left", Down= "down", Up= "down",
	}
	local roll_buttons= {
		Left= "left", Right= "left", Down= "down", Up= "up",
	}
	local parts_per_beat= 48
	local mine_state_map= {
		parts_per_beat= 1, quanta= {{per_beat= 1, states= {1}}}}
	local active_state_map= {
		parts_per_beat= parts_per_beat, quanta= {
			{per_beat= 1, states= {1, 2}}, -- 4th
			{per_beat= 2, states= {5, 6}}, -- 8th
			{per_beat= 3, states= {9, 10}}, -- 12th
			{per_beat= 4, states= {13, 14}}, -- 16th
			{per_beat= 6, states= {17, 18}}, -- 24th
			{per_beat= 8, states= {21, 22}}, -- 32nd
			{per_beat= 12, states= {25, 26}}, -- 48th
			{per_beat= 16, states= {29, 30}}, -- 64th
		},
	}
	local inactive_quanta= {}
	for i, quantum in ipairs(active_state_map.quanta) do
		local states= {}
		for s, state in ipairs(quantum.states) do
			states[s]= state + 2
		end
		inactive_quanta[i]= {per_beat= quantum.per_beat, states= states}
	end
	local inactive_state_map= {parts_per_beat= parts_per_beat, quanta= inactive_quanta}
	local columns= {}
	for i, button in ipairs(button_list) do
		local hold_tex= hold_buttons[button] .. "_hold 8x4.png"
		local roll_tex= roll_buttons[button] .. "_roll 8x4.png"
		columns[i]= {
			taps= {
				NewSkinTapPart_Tap= {
					state_map= tap_state_map,
					actor= Def.Sprite{Texture= tap_graphic,
						-- Use the InitCommand to rotate the arrow appropriately.
						InitCommand= function(self) if rotate_taps then self:rotationz(rots[button]) end end}},
				NewSkinTapPart_Mine= {
					state_map= mine_state_map,
					actor= Def.Sprite{Texture= "mine.png"}},
				NewSkinTapPart_Lift= { -- fuck lifts
					state_map= mine_state_map,
					actor= Def.Sprite{Texture= "mine.png"}},
			},
			holds= {
				TapNoteSubType_Hold= {
					{
						state_map= inactive_state_map,
						textures= {hold_tex},
						flip= hold_flips[button],
						disable_filtering= false,
					},
					{
						state_map= active_state_map,
						textures= {hold_tex},
						flip= hold_flips[button],
					},
				},
				TapNoteSubType_Roll= {
					{
						state_map= inactive_state_map,
						textures= {roll_tex},
						flip= roll_flips[button],
					},
					{
						state_map= active_state_map,
						textures= {roll_tex},
						flip= roll_flips[button],
					},
				},
			},
			reverse_holds= {
				TapNoteSubType_Hold= {
					{
						state_map= inactive_state_map,
						textures= {hold_tex},
						flip= rev_hold_flips[button],
					},
					{
						state_map= active_state_map,
						textures= {hold_tex},
						flip= rev_hold_flips[button],
					},
				},
				TapNoteSubType_Roll= {
					{
						state_map= inactive_state_map,
						textures= {roll_tex},
						flip= rev_roll_flips[button],
					},
					{
						state_map= active_state_map,
						textures= {roll_tex},
						flip= rev_roll_flips[button],
					},
				},
			},
		}
	end
	return {
		columns= columns,
		vivid_operation= true, -- output 200%
	}
end
